from helper_functions import *
import math

# A table is just a list of lists. The first element of the list is
# a list of the labels for the fields in the table (i.e. ["name","email"]).
# The rest of the elements of the list correspond to the entries in the
# table. All entries (including the list of lables) must all have the same
# number of elements.

# Make a table from a field list and a set of entries
def make_table(fieldlist,entries):
    return [fieldlist] + entries

# Make a table from just a field list
def make_new_table(fieldlist):
    return [fieldlist]

# Get the fields of a table "t"
def table_fields(t):
    return t[0]

# Get the entries of a table "t"
def table_entries(t):
    return t[1:]

# Add a new entry to a table
def table_insert(table,entry):
    # We first need to check that the new entry has as many
    # elements as the other entries.  If it doesn't, we
    # throw an error.
    if len(entry) != len(table_fields(table)):
        raise StandardError("New table entry has " + str(len(entry)) + " values, but " + str(len(table_fields)) + "were expected")

    # Add the new element. The "append" function here is
    # the mutation equivalent of (table + [entry])
    table.append(entry)

# Finds which column in the table holds the data for a given field
# For example, table_field_number(bids,"item_name") will return 1
def table_field_number(table,field):
    return list_find_element(table_fields(table),field)

# The following code handles the printing out of the tables. Understanding this
# code is not crucial to doing the problem set, but it's good practice.

display_width = 80  # How many characters wide each entry is

# This function takes a given string and makes it have "length" number
# of characters.  If it's too short it will get padded with spaces.  If it's
# too long, we truncate it.  This will make it so all of our columns in the
# table are the same size.
def make_string_length(s,length):
    s = str(s)
    if len(s) > length:
        return s[0:length]
    else:
        return s + (" " * (length - len(s)))

# print_list_width_helper combines all of the strings (one for each column)
# that make up a given entry in the table. This is passed back to
# print_list_width for printing.
def print_list_width_helper(lst,fieldwidth):
    if len(lst) == 0:
        return ""
    else:
        # Call make_string_length to make sure each element is the right size
        return make_string_length(lst[0],fieldwidth) + print_list_width_helper(lst[1:],fieldwidth)

# Given a list and a number of characters "fieldwidth", print_list_width
# prints out the elements of the list on a single line.  Each column takes
# up "fieldwidth" number of characters.
def print_list_width(lst,fieldwidth):
    print print_list_width_helper(lst,fieldwidth)

# Prints out a table (both the headers and the entries) using the
# helper functions we just defined.
def table_display(table):
    # Calculate the column size (display_width/number of columns)
    fieldwidth = int(math.floor(display_width/len(table_fields(table))))
    # Print out the headers
    print_list_width(table_fields(table),fieldwidth)
    # Print out a line
    print_list_width(["------------------------" for x in table_fields(table)],fieldwidth)
    # Print out each element
    map(lambda x: print_list_width(x,fieldwidth),table_entries(table))
